// TODO: This should be its own crate
use std::fmt::Display;

pub mod eval;
pub mod parser;
pub mod tokenizer;

// TODO: Non floating point arithmetic please
pub type Number = f64;

#[derive(Debug, PartialEq, PartialOrd, Clone)]
pub enum MathExpr<'a> {
    Number(Number),
    Sym(&'a str),
    UnOp(UnMathOp, Box<MathExpr<'a>>),
    BinOp(BinMathOp, Box<MathExpr<'a>>, Box<MathExpr<'a>>),
}

impl<'a> Display for MathExpr<'a> {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        match self {
            MathExpr::Number(n) => write!(f, "{n}"),
            MathExpr::Sym(s) => write!(f, "{s}"),
            MathExpr::UnOp(op, expr) => write!(f, "{op}{expr}"),
            MathExpr::BinOp(op, lhs, rhs) => write!(f, "({lhs}{op}{rhs})"),
        }
    }
}

#[derive(Debug, PartialEq, PartialOrd, Clone)]
pub enum UnMathOp {
    Minus,
}

impl Display for UnMathOp {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        match self {
            UnMathOp::Minus => f.write_str("-"),
        }
    }
}

#[derive(Debug, PartialEq, PartialOrd, Clone)]
pub enum BinMathOp {
    Add,
    Sub,
    Mult,
    Div,
    // TODO: Integer division
    // IntegerDiv,
    Pow,
}

impl Display for BinMathOp {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        match self {
            BinMathOp::Add => f.write_str("+"),
            BinMathOp::Sub => f.write_str("-"),
            BinMathOp::Mult => f.write_str("*"),
            BinMathOp::Div => f.write_str("/"),
            BinMathOp::Pow => f.write_str("^"),
        }
    }
}

#[derive(Debug, PartialEq, PartialOrd, Clone, Copy)]
pub enum MathToken<'a> {
    Number(Number),
    Sym(&'a str),
    LParen,
    RParen,
    Plus,
    Minus,
    Slash,
    Star,
    DoubleStar,
    Caret,
}

impl<'a> Display for MathToken<'a> {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        match self {
            MathToken::Number(n) => write!(f, "{n}"),
            MathToken::Sym(s) => write!(f, "{s}"),
            MathToken::LParen => f.write_str("("),
            MathToken::RParen => f.write_str(")"),
            MathToken::Plus => f.write_str("+"),
            MathToken::Minus => f.write_str("-"),
            MathToken::Slash => f.write_str("/"),
            MathToken::Star => f.write_str("*"),
            MathToken::DoubleStar => f.write_str("**"),
            MathToken::Caret => f.write_str("^"),
        }
    }
}

impl<'a> MathToken<'a> {
    pub fn precedence(&self) -> u8 {
        match self {
            MathToken::Number(_) => 0,
            MathToken::Sym(_) => 0,
            MathToken::LParen => 0,
            MathToken::RParen => 0,
            MathToken::Plus => 1,
            MathToken::Minus => 1,
            MathToken::Slash => 2,
            MathToken::Star => 2,
            MathToken::DoubleStar => 3,
            MathToken::Caret => 3,
        }
    }
}
