use gtk::{Orientation, pango::EllipsizeMode, prelude::BoxExt};

use crate::modules::SearchResult;

pub fn create(container: &gtk::Box, result: &Box<dyn SearchResult>, is_selected: bool) {
    let res_box = gtk::Box::builder()
        .orientation(Orientation::Horizontal)
        .css_classes(if is_selected {
            vec!["result", "selected"]
        } else {
            vec!["result"]
        })
        .spacing(8)
        .build();

    // If there is an icon display it, otherwise display an empty box.
    match result.icon() {
        Some(icon) => res_box.append(&gtk::Image::builder().gicon(&icon).pixel_size(16).build()),
        None => res_box.append(&gtk::Box::new(Orientation::Horizontal, 0)),
    }

    let detail_box = gtk::Box::new(Orientation::Vertical, 2);
    detail_box.append(
        &gtk::Label::builder()
            .label(result.label())
            .css_classes(vec!["result-name"])
            .halign(gtk::Align::Start)
            .max_width_chars(40)
            .ellipsize(EllipsizeMode::End)
            .hexpand(true)
            .build(),
    );
    res_box.append(&detail_box);

    container.append(&res_box);
}
